from time import sleep

import archr
import unittest

from common import build_container

BIN_CAT = "/bin/cat"
STRACE_ARGS = "-f".split()


class TestAnalyzerStrace(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        build_container("cat")
        build_container("socat")

    def check_strace_proc(self, t, **kwargs):  # pylint:disable=no-self-use
        b = archr.analyzers.STraceAnalyzer(t)
        trace = b.fire(args_suffix=["/etc/passwd"], trace_args=STRACE_ARGS, **kwargs).splitlines()
        assert any(b"open" in t and b"passwd" in t for t in trace)
        assert any(b"read" in t and b"root" in t for t in trace)
        assert any(b"write" in t and b"root" in t for t in trace)

    def check_strace_attach(self, t, **kwargs):  # pylint:disable=no-self-use
        target = t.run_command()  # start target
        b = archr.analyzers.STraceAttachAnalyzer(t)
        pid = target.pid if isinstance(t, archr.targets.LocalTarget) else t.get_proc_pid("socat")
        with b.fire_context(pid=pid, trace_args=STRACE_ARGS, **kwargs) as flight:
            sleep(2)
            nc = flight.get_channel("tcp:0")
            nc.send(b"ahoi!")
            assert nc.readuntil(b"ahoi!", timeout=5) == b"ahoi!"
            nc.close()
            target.terminate()

        trace = flight.result.splitlines()
        assert any(b"read" in t and b"ahoi" in t for t in trace)
        assert any(b"write" in t and b"ahoi" in t for t in trace)

    def test_strace_proc_local(self):
        with archr.targets.LocalTarget(["/bin/cat"]).build().start() as t:
            self.check_strace_proc(t)

    def test_strace_proc_docker(self):
        with archr.targets.DockerImageTarget("archr-test:cat").build().start() as t:
            self.check_strace_proc(t)

    def test_strace_attach_local(self):
        with archr.targets.LocalTarget(
            "socat tcp-l:9137,reuseaddr exec:cat".split(), tcp_ports=[9137]
        ).build().start() as t:
            self.check_strace_attach(t)

    def test_strace_attach_docker(self):
        with archr.targets.DockerImageTarget("archr-test:socat").build().start() as t:
            self.check_strace_attach(t)


if __name__ == "__main__":
    unittest.main()



import tempfile
import struct
import shutil
import archr
import os
import unittest

from common import build_container


class TestShellcode(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        build_container("entrypoint-env")
        build_container("vuln_stacksmash")

    def shellcode_checks(self, t):
        b = archr.analyzers.DataScoutAnalyzer(t)
        with t.shellcode_context(asm_code=b.exit_shellcode(exit_code=123)) as p:
            stdout, _ = p.communicate()
            assert p.wait() == 123

        with t.shellcode_context(asm_code=b.echo_shellcode("TESTING THIS THING!")) as p:
            stdout, _ = p.communicate()
            assert stdout == b"TESTING THIS THING!"

        with t.shellcode_context(asm_code=b.echo_shellcode("TESTING THIS THING!") + b.exit_shellcode()) as p:
            stdout, _ = p.communicate()
            assert stdout == b"TESTING THIS THING!"
            assert p.wait() == 42

        with t.shellcode_context(asm_code=b.read_file_shellcode("/proc/self/cmdline")) as p:
            stdout, _ = p.communicate()
            assert stdout == t.target_path.encode("utf-8") + b"\0"

    def test_shellcode_amd64(self):
        with archr.targets.DockerImageTarget("archr-test:entrypoint-env").build().start() as t:
            self.shellcode_checks(t)

    def test_shellcode_i386(self):
        with archr.targets.DockerImageTarget("archr-test:vuln_stacksmash", target_arch="i386").build().start() as t:
            self.shellcode_checks(t)


class TestDatascout(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        build_container("entrypoint-env")

    def datascout_checks(self, t):
        b = archr.analyzers.DataScoutAnalyzer(t)
        argv, env, aux, maps = b.fire()

        ld_addr = next(addr for name, addr in maps.items() if "linux-gnu/ld-" in name)

        assert argv == [a.encode("utf-8") for a in t.target_args]
        assert b"ARCHR=YES" in env
        assert ld_addr in struct.unpack("<%dQ" % (len(aux) / 8), aux)
        return argv, env, aux, maps

    def test_datascout(self):
        with archr.targets.DockerImageTarget("archr-test:entrypoint-env").build().start() as t:
            _, _, _, maps = self.datascout_checks(t)
            assert (
                next(v for k, v in maps.items() if k.startswith("/usr/lib/x86_64-linux-gnu/libc-")) & 0xFFFFFF000000
                == 0x7FFFF7000000
            )
            assert (
                next(v for k, v in maps.items() if k.startswith("/usr/lib/x86_64-linux-gnu/ld-")) & 0xFFFFFF000000
                == 0x7FFFF7000000
            )
            assert maps["[stack-end]"] == 0x7FFFFFFFF000
            assert maps["[vsyscall]"] == 0xFFFFFFFFFF600000
            assert maps["[vdso]"] & 0xFFFFFF000000 == 0x7FFFF7000000
            assert maps["[vvar]"] & 0xFFFFFF000000 == 0x7FFFF7000000
            assert maps["[heap]"] == 0x555555560000

    def test_datascout_local(self):
        # copy to a writable location
        tf = tempfile.mktemp()
        shutil.copy("/usr/bin/env", tf)
        with archr.targets.LocalTarget([tf], target_env=["ARCHR=YES"]).build().start() as t:
            _, _, _, maps = self.datascout_checks(t)
            local_maps_expected = [
                "linux-gnu/libc-",
                "linux-gnu/ld-",
                "[stack-end]",
                "[heap]",
                "[vvar]",
                "[vdso]",
                "[vsyscall]",
            ]
            for map_expected in local_maps_expected:
                assert any(map_expected in name for name in maps)

        os.unlink(tf)


class TestStackSmash(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        build_container("vuln_stacksmash")

    # 32-bit vuln_stacksmash
    def test_stacksmash(self):
        with archr.targets.DockerImageTarget("archr-test:vuln_stacksmash", target_arch="i386").build().start() as t:
            b = archr.analyzers.DataScoutAnalyzer(t)
            argv, env, aux, maps = b.fire()

            assert b"PWD=/" in env
            assert maps["/usr/lib/i386-linux-gnu/ld-2.31.so"] in struct.unpack("<%dI" % (len(aux) / 4), aux)
            assert "[stack-end]" in maps
            assert "[heap]" in maps
            assert "[vvar]" in maps
            assert "[vdso]" in maps


if __name__ == "__main__":
    unittest.main()



from ..errors import ArchrError
from . import ContextAnalyzer
import contextlib
import subprocess
import tempfile
import logging
import signal
import shutil
import os
import time

l = logging.getLogger("archr.analyzers.gdb")


class FakeTempdir:
    def __init__(self, path):
        self.name = path

    def cleanup(self):
        return


class GDBResult:
    returncode = None
    signal = None
    crashed = False
    timed_out = False

    def __init__(self, trace_dir=None):
        if trace_dir is None:
            self.trace_dir = tempfile.TemporaryDirectory(prefix="gdb_trace_dir_")
        else:
            self.trace_dir = FakeTempdir(trace_dir)


class GDBAnalyzer(ContextAnalyzer):
    REQUIRED_IMPLANT = "gdb"

    def __init__(self, target, local_trace_dir=None, timeout=10):
        super().__init__(target)
        self.timeout = timeout
        self.local_trace_dir = local_trace_dir

    @contextlib.contextmanager
    def fire_context(self, prefix_args=None, gdb_args=None, gdb_script=None, sleep_time=0.1):
        """Run the target with gdb.

        Keyword arguments:
        prefix_args -- additional commands BEFORE the gdb command (default None)
        gdb_args -- addition args for gdb (default None)
        gdb_script -- Path of an optional gdb_script file (default None)
        """

        if self.local_trace_dir:
            if os.path.exists(self.local_trace_dir):
                shutil.rmtree(self.local_trace_dir)
            os.mkdir(self.local_trace_dir)
        else:
            self.local_trace_dir = tempfile.mkdtemp(prefix="/tmp/gdb_tracer_")

        fire_path = os.path.join(self.target.tmpwd, "gdb", "fire")
        gdb_command = []
        if prefix_args:
            gdb_command += prefix_args
        gdb_command += [fire_path]
        if gdb_args:
            gdb_command += gdb_args
        if gdb_script:
            paths = {}
            d_src = os.path.dirname(gdb_script)
            d_dst = os.path.dirname(fire_path)
            paths[d_dst] = d_src
            self.target.inject_paths(paths)
            script_remote_path = os.path.join(d_dst, os.path.basename(gdb_script))
            gdb_command += ["-x", script_remote_path]
            gdb_command += ["--args"]
            gdb_command += self.target.target_args

        r = GDBResult(trace_dir=self.local_trace_dir)
        try:
            with self.target.flight_context(gdb_command, timeout=self.timeout, result=r) as flight:
                # TODO: we need a better way of dealing with this, dnsmasq is too slow at initializing
                time.sleep(sleep_time)
                yield flight
        except subprocess.TimeoutExpired:
            r.timed_out = True
        else:
            r.timed_out = False

            r.returncode = flight.process.returncode
            assert r.returncode is not None

            # did a crash occur?
            if r.returncode in [139, -11]:
                r.crashed = True
                r.signal = signal.SIGSEGV
            elif r.returncode == [132, -9]:
                r.crashed = True
                r.signal = signal.SIGILL
